#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,W0231
"""Driver for LCD. Uses i2c. Chip 8574,
Simpe wiring:
GPIO    PCF8574
GND     A0
GND     A1
GND     A2
GND     GND
+5V     Vcc
SDA     SDA
SCL     SCL

PCF8574     LCD
P4          LCD4 (RS)
P5          LCD6 (E)
P6          E2
P3          LCD14 (DB7)
P2          LCD13 (DB6)
P1          LCD12 (DB5)
P0          LCD11 (DB4)

"""

import time
import smbus  # pylint: disable=I0011,F0401
from charlcd.drivers.base import BaseDriver


class I2C(BaseDriver):
    """I2C LCD driver"""
    def __init__(self, address, port):
        """
        Set address port, pins
        Args:
            address: hex i2c address
            port: bus number
        """
        self.address = address
        self.port = port
        self.mode = 8
        self.pins = {
            'RS': 4,
            'E': 5,
            'E2': None,  # 6
            'DB4': 0,
            'DB5': 1,
            'DB6': 2,
            'DB7': 3
        }
        self.bus = smbus.SMBus(port)
        self.char_buffer = None
        self.initialized = False

    def init(self):
        """recalculate pins to values"""
        if self.initialized:
            return
        pins = {}
        for k in self.pins:
            if self.pins[k] is not None:
                pins[k] = int(pow(2, self.pins[k]))
        self.pins = pins

        self.initialized = True

    def cmd(self, char, enable=0):
        """
        Send command to lcd
        Args:
            char: hex command to write
        """
        if self.mode == 8:
            self.prepare_send(char & 0x0F, enable)
        else:
            self.prepare_send(char >> 4, enable)
            self.prepare_send(char & 0x0F, enable)

    def shutdown(self):
        """shutdown routine"""
        pass

    def prepare_send(self, char, enable=0):
        """
        Prepares data for send and call send
            char: char to write
            enable: line to use
        """
        self.char_buffer = char
        self.send(enable)

    def send(self, enable=0):
        """
        Send enable signal
        Args:
            enable: line to use
        """
        if enable == 0:
            pin = self.pins['E']
        elif enable == 1:
            pin = self.pins['E2']

        if pin is None:
            raise IndexError("Wrong enable index")

        self.write(self.char_buffer | pin)
        time.sleep(0.005)
        self.write(self.char_buffer & (0xFF - pin))

    def write(self, byte, enable=0):
        """
        Write data to bus
        Args:
            byte: hex to write to i2c bus
        """
        self.bus.write_byte(self.address, byte)

    def char(self, char, enable=0):
        """
        Send char
        Args:
            char: char to swrite
            enable: enable line
        """
        char = ord(char)
        if self.mode == 8:
            self.prepare_send(self.pins['RS'] | (char & 0x0F), enable)
        else:
            self.prepare_send(self.pins['RS'] | (char >> 4), enable)
            self.prepare_send(self.pins['RS'] | (char & 0x0F), enable)

    def set_mode(self, mode):
        """
        Set working mode (4/8-bit)
        Args:
            mode: mode to set
        """
        self.mode = mode



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,R0913,R0902
"""Interface for buffered inputs
Provide functions that must be implemented by and buffered lcd
"""


class Buffered(object):
    """Required functions for buffered lcd driver"""
    def init(self):
        """screen and buffer init"""
        raise NotImplementedError("init not implemented")

    def buffer_clear(self, from_x=None, from_y=None, width=None, height=None):
        """clears buffer"""
        raise NotImplementedError("buffer_clear not implemented")

    def flush(self):
        """Flush buffer to screen, skips chars that didn't change"""
        raise NotImplementedError("flush not implemented")



#!/usr/bin/python
# -*- coding: utf-8 -*-
#pylint: disable=I0011,W0231

"""Keyboard control"""

__author__ = 'Bartosz Kościów'
__credits__ = "http://stackoverflow.com/questions/510357/" \
              "python-read-a-single-character-from-the-user"

import socket
import termios
import sys, tty

TCP_IP = '127.0.0.1'
TCP_PORT = 5005
BUFFER_SIZE = 1024


def getch():
    """get char from io buffer"""
    file_description = sys.stdin.fileno()
    old_settings = termios.tcgetattr(file_description)
    try:
        tty.setraw(file_description)
        char = sys.stdin.read(1)
    finally:
        termios.tcsetattr(file_description, termios.TCSADRAIN, old_settings)

    if char == '\x03':
        raise KeyboardInterrupt
    elif char == '\x04':
        raise EOFError
    return char


def keyboard_main():
    """main loop"""
    try:
        conn_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        conn_socket.connect((TCP_IP, TCP_PORT))
        while True:
            key = getch()
            if key == 'a':
                conn_socket.send('player.move.left')
            if key == 'd':
                conn_socket.send('player.move.right')
            if key == ' ':
                conn_socket.send('player.move.fire')

    finally:
        conn_socket.close()

if __name__ == '__main__':
    keyboard_main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Simple key read"""
from builtins import object
__author__ = 'Bartosz Kościów'

import socket
import termios, fcntl, sys, os

TCP_IP = '127.0.0.1'
TCP_PORT = 5005
BUFFER_SIZE = 1024


class Keyboard(object):
    """Read keyboard"""
    def __init__(self):
        self.conn_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.conn_socket.connect((TCP_IP, TCP_PORT))
        self.file_descriptior = sys.stdin.fileno()

        self.oldterm = termios.tcgetattr(self.file_descriptior)
        self.newattr = termios.tcgetattr(self.file_descriptior)
        self.newattr[3] = self.newattr[3] & ~termios.ICANON & ~termios.ECHO
        termios.tcsetattr(self.file_descriptior, termios.TCSANOW, self.newattr)

        self.oldflags = fcntl.fcntl(self.file_descriptior, fcntl.F_GETFL)
        fcntl.fcntl(
            self.file_descriptior,
            fcntl.F_SETFL,
            self.oldflags | os.O_NONBLOCK
        )

    def read(self):
        """read one key from buffer if avaible"""
        try:
            key = sys.stdin.read(1)
            if key == 'a':
                self.send('player.move.left')
            if key == 'd':
                self.send('player.move.right')
            if key == ' ':
                self.send('player.move.fire')
        except IOError:
            pass

    def send(self, event):
        """send event to game"""
        self.conn_socket.send(event.encode('UTF-8'))

    def shutdown(self):
        """restore console"""
        termios.tcsetattr(
            self.file_descriptior,
            termios.TCSAFLUSH,
            self.oldterm
        )
        fcntl.fcntl(self.file_descriptior, fcntl.F_SETFL, self.oldflags)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,W0231
"""Dummy lcd driver. Used in tests"""

from charlcd.drivers.base import BaseDriver


class Null(BaseDriver):
    """Dummy LCD driver"""
    def __init__(self):
        """empty init class"""
        self.pins = {
            'RS': 4,
            'E': 5,
            'E2': None,
            'DB4': 0,
            'DB5': 1,
            'DB6': 2,
            'DB7': 3
        }
        return

    def init(self):
        """init driver"""
        return

    def cmd(self, char, enable=0):
        """send command to lcd"""
        return char

    def shutdown(self):
        """shutdown steps"""
        return

    def send(self, enable=0):
        """send data to lcd"""
        return

    def write(self, char, enable=0):
        """write to lcd"""
        return char

    def char(self, char, enable=0):
        """send char to lcd"""
        return ord(char)

    def set_mode(self, mode):
        """set lcd mode"""
        return mode



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,R0913
"""Virtual Lcd class for direct input

Example:
lcd = CharLCD(20,4, I2C(0x20, 1))
lcd = CharLCD(20,4, GPIO())
lcd = CharLCD(20,4, I2C(0x20, 1), 1, 1)
lcd = CharLCD(20,4, I2C(0x20, 1), cursor_blink=1)
"""

import charlcd.abstract.lcd_virtual as lcd
import charlcd.abstract.direct_interface as direct


class CharLCD(direct.Direct, lcd.CharLCDVirtual):
    """Virtual Lcd class for direct input"""
    def __init__(self, width, height):
        lcd.CharLCDVirtual.__init__(self,
                                    width, height,
                                    lcd.DISPLAY_MODE_DIRECT)

    def init(self):
        """Buffer init"""
        if self.initialized:
            return
        if len(self.displays) == 0:
            raise ValueError("Add lcd before init vlcd")

        for display in self.displays:
            display['lcd'].init()
        self.initialized = True

    def write(self, string, pos_x=None, pos_y=None):
        """Write string to lcd
        Args:
            string: string to display
            pos_x: x coords
            pos_y: y coords
        """
        if pos_x is not None and pos_y is not None:
            self.set_xy(pos_x, pos_y)

        for char in string:
            self._char(char)

    def _char(self, char):
        """Write char
        Args:
            char: char to display
        """
        if self.current_pos['x'] >= self.width:
            return

        self._select_active_display()

        self.current_pos['x'] += 1
        if self.active_display is None:
            return

        self.active_display['lcd'].write(char)

        if self.current_pos['x'] >= self.active_display['x'] + \
                self.active_display['width'] - 1 - \
                self.active_display['offset_x']:
            self.active_display = None

    def set_xy(self, pos_x, pos_y):
        """Set cursor position @ (x, y)
        Args:
            pos_x: x position
            pos_y: y position
        """
        lcd.CharLCDVirtual.set_xy(self, pos_x, pos_y)
        self.active_display = None

    def _stream_char(self, char):
        """Stream char on screen, following chars are put one after another.
        Restart from beginning after reaching end
        Args:
            char: char to display
        """
        self._char(char)
        self._next_cursor_position()

    def stream(self, string):
        """Stream string - use stream_char
        Args:
            string: string to display
        """
        for char in string:
            self._stream_char(char)

    def _next_cursor_position(self):
        """Calculate next position, break lines and can go back to top"""
        if self.current_pos['x'] >= self.width:
            self.current_pos['x'] = 0
            self.current_pos['y'] += 1
            self.active_display = None
            if self.current_pos['y'] >= self.height:
                self.current_pos['y'] = 0

    def _select_active_display(self):
        """Helper,
        looks for proper lcd for given coordinates, may return None
        """
        if self.active_display is None:
            self.active_display = self.get_display(
                self.current_pos['x'],
                self.current_pos['y']
            )
            if self.active_display is None:
                return
            self.active_display['lcd'].set_xy(
                self.current_pos['x'] - self.active_display['x'] +
                self.active_display['offset_x'],
                self.current_pos['y'] - self.active_display['y'] +
                self.active_display['offset_y']
            )



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""test script for buffered lcd"""

import sys
sys.path.append("../../")
import RPi.GPIO as GPIO  # NOQA pylint: disable=I0011,F0401
from charlcd import buffered as lcd  # NOQA
from charlcd.drivers.gpio import Gpio  # NOQA
from charlcd.drivers.i2c import I2C  # NOQA

GPIO.setmode(GPIO.BCM)


def test1():
    """demo 20x4 and 16x2"""
    lcd_2 = lcd.CharLCD(20, 4, Gpio())
    lcd_2.init()
    lcd_2.write('-  Blarg !')
    lcd_2.write('-   Grarg !', 0, 1)
    lcd_2.write('-    ALIVE  !!!!', 0, 2)

    lcd_2.flush()

    lcd_2.write('-    ALIVE  !!!!.', 0, 2)
    lcd_2.flush()

    lcd_1 = lcd.CharLCD(16, 2, I2C(0x20, 1), 0, 0)
    lcd_1.init()
    lcd_1.write('-!Second blarg!')
    lcd_1.write("-second line", 0, 1)
    lcd_1.flush()


def test2():
    """demo 40x4"""
    drv = Gpio()
    drv.pins['E2'] = 10
    drv.pins['E'] = 24
    lcd_1 = lcd.CharLCD(40, 4, drv, 0, 0)
    lcd_1.init()
    lcd_1.write('-  Blarg !')
    lcd_1.write('-   Grarg !', 0, 1)
    lcd_1.write('-    ALIVE  !!!!', 0, 2)
    lcd_1.flush()

    lcd_1.write('/* ', 19, 0)
    lcd_1.write('|*|', 19, 1)
    lcd_1.write(' */', 19, 2)

    lcd_1.flush()


def test3():
    """demo 16x2"""
    lcd_1 = lcd.CharLCD(16, 2, I2C(0x20, 1), 0, 0)
    lcd_1.init()
    lcd_1.set_xy(10, 0)
    lcd_1.stream("1234567890qwertyuiopasdfghjkl")
    lcd_1.flush()


test1()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,R0913,R0902

"""Interface for direct inputs
Provide functions that must be implemented by and direct lcd
"""


class Direct(object):
    """Required functions for direct lcd driver"""
    def stream(self, string):
        """Stream string - use stream_char
        Args:
            string: string to display
        """
        raise NotImplementedError("stream_string not implemented")



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,R0913,R0902
"""Main lcd class"""

import time

LINE0 = 0x80
LINE1 = 0xC0
LINE2 = 0x94
LINE3 = 0xD4
LCD_LINES = [
    LINE0,
    LINE1,
    LINE2,
    LINE3
]

DISPLAY_MODE_DIRECT = 'direct'
DISPLAY_MODE_BUFFERED = 'buffered'


class CharLCD(object):
    """Class for char LCDs
        Shouldn't be instanced - use lcd_buffered or lcd_direct
    """
    def __init__(self, width, height, driver, display_mode=DISPLAY_MODE_DIRECT,
                 cursor_visible=1, cursor_blink=1):
        """Inits lcd class
        Args:
            width (int): width of lcd
            height (int): height of lcd
            driver: driver to use for lcd communication (i2c, gpio, null)
            display_mode: automatically set by calling
                lcd_direct or lcd_buffered
            cursor_visible: set show cursor
            cursor_blink: set cursor blink
        """
        self.width = width
        self.height = height
        self.driver = driver
        self.current_pos = {
            'x': 0,
            'y': 0
        }
        self.display_mode = display_mode
        self.cursor_blink = cursor_blink
        self.cursor_visible = cursor_visible
        self.twin_lcd = False  # are we using 40x4 = 2* 20x4 lcd ?
        if self.driver.pins['E2'] is not None:
            self.twin_lcd = True

    def init(self):
        """Inits lcd display, send commands"""
        self.driver.init()
        self._init(0)
        if self.is_twin():
            self._init(1)

    def _init(self, enable):
        """subroutine to init lcd"""
        self.driver.cmd(3, enable)
        time.sleep(0.05)
        self.driver.cmd(3, enable)
        time.sleep(0.05)
        self.driver.cmd(3, enable)
        time.sleep(0.05)
        self.driver.cmd(2, enable)
        self.driver.set_mode(4)
        self.driver.cmd(0x28, enable)
        self.driver.cmd(0x08, enable)
        self.driver.cmd(0x01, enable)
        self.driver.cmd(0x06, enable)
        self.driver.cmd(12 +
                        (self.cursor_visible * 2) +
                        (self.cursor_blink * 1), enable)

    def get_width(self):
        """returns lcd width"""
        return self.width

    def get_height(self):
        """return lcd height"""
        return self.height

    def get_display_mode(self):
        """return display mode, direct or buffered"""
        return self.display_mode

    def shutdown(self):
        """call shutdown on driver"""
        self.driver.shutdown()

    def write(self, string, pos_x=None, pos_y=None):
        """Writes content at position(x,y) or current
        Will change internal position marker to reflect string write
        Args:
            content: content to write
            pos_x: x position
            pos_y: y position
        """
        raise NotImplementedError("string not implemented")

    def set_xy(self, pos_x, pos_y):
        """Set cursor position to (x, y)
        Args:
            pos_x: x position
            pos_y: y position
        """
        if pos_x >= self.width or pos_x < 0:
            raise IndexError
        if pos_y >= self.height or pos_y < 0:
            raise IndexError
        self.current_pos['x'] = pos_x
        self.current_pos['y'] = pos_y

    def get_xy(self):
        """return current cursor position"""
        return self.current_pos

    def stream(self, string):
        """Stream string
        Args:
            string: string to display
        """
        raise NotImplementedError("stream not implemented")

    def is_twin(self):
        """returns true if we are using twin lcd"""
        return self.twin_lcd

    def get_line_address(self, pos_y=None):
        """Return start hex address for line
        Args:
            pos_y: line number
        """
        if pos_y is None:
            pos_y = self.current_pos['y']

        if pos_y >= self.height or pos_y < 0:
            raise IndexError

        if not self.is_twin() or pos_y < 2:
            return LCD_LINES[pos_y]

        return LCD_LINES[pos_y - 2]

    def _get_enable(self, pos_y=None):
        """get proper enable line
        Args:
            pos_y: line number
        """
        if pos_y is None:
            pos_y = self.current_pos['y']
        if self.is_twin() and pos_y > 1:
            enable = 1
        else:
            enable = 0

        return enable



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011
"""
Class interface with functions required by driver implementation
"""


class BaseDriver(object):
    """Simple interface"""
    def __init__(self):
        """Class init"""
        raise NotImplementedError("__init__ not implemented")

    def init(self):
        """dummy init function"""
        raise NotImplementedError("init not implemented")

    def cmd(self, char, enable=0):
        """write command"""
        raise NotImplementedError("cmd not implemented")

    def shutdown(self):
        """shutdown procedure"""
        raise NotImplementedError("shutdown not implemented")

    def send(self, enable=0):
        """send ack command"""
        raise NotImplementedError("send not implemented")

    def write(self, char, enable=0):
        """write data to lcd"""
        raise NotImplementedError("write not implemented")

    def char(self, char, enable=0):
        """write char to lcd"""
        raise NotImplementedError("char not implemented")

    def set_mode(self, mode):
        """sets driver mode. 4/8 bit"""
        raise NotImplementedError("set_mode not implemented")



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,R0913
"""Lcd class for direct input

Example:
lcd = CharLCD(20,4, I2C(0x20, 1))
lcd = CharLCD(20,4, GPIO())
lcd = CharLCD(20,4, I2C(0x20, 1), 1, 1)
lcd = CharLCD(20,4, I2C(0x20, 1), cursor_blink=1)
"""

import charlcd.abstract.lcd as lcd
import charlcd.abstract.direct_interface as direct_interface


class CharLCD(direct_interface.Direct, lcd.CharLCD):
    """Class for char LCDs
    Uses direct input. Operates directly on lcd. What you write is what you see
    """
    def __init__(self, width, height, driver,
                 cursor_visible=1, cursor_blink=1):
        lcd.CharLCD.__init__(self, width, height, driver,
                             lcd.DISPLAY_MODE_DIRECT,
                             cursor_visible, cursor_blink)

    def write(self, string, pos_x=None, pos_y=None):
        """Write string to lcd
        Args:
            string: string to display
            pos_x: x coords
            pos_y: y coords
        """
        if pos_x is not None and pos_y is not None:
            self.set_xy(pos_x, pos_y)

        for char in string:
            self._char(char)
            if self.current_pos['x'] >= self.width:
                return

    def _char(self, char):
        """Write char
        Args:
            char: char to display
        """
        self.driver.char(char, self._get_enable())
        self.current_pos['x'] += 1

    def _position(self, pos, enable=0):
        """Set cursor position
        Args:
            pos: hex address for new cursor position
        """
        self.driver.cmd(pos, enable)

    def set_xy(self, pos_x, pos_y):
        """Set cursor position by X Y. it recalculate position to hex address
        Args:
            pos_x: x position
            pos_y: y position
        """
        lcd.CharLCD.set_xy(self, pos_x, pos_y)
        self._position(
            self.get_line_address(pos_y) + pos_x,
            self._get_enable()
        )

    def stream(self, string):
        """Stream string - use stream_char
        Args:
            string: string to display
        """
        for char in string:
            self._stream_char(char)

    def _stream_char(self, char):
        """Stream char on screen, following chars are put one after another.
        Restart from beginning after reaching end
        Args:
            char: char to display
        """
        self._char(char)
        self._next_cursor_position()

    def _next_cursor_position(self):
        """calculate next cursor position"""
        if self.current_pos['x'] >= self.width:
            self.current_pos['x'] = 0
            self.current_pos['y'] += 1
            if self.current_pos['y'] >= self.height:
                self.current_pos['y'] = 0
            self.set_xy(0, self.current_pos['y'])



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""Simple game"""
from __future__ import division
from future import standard_library
standard_library.install_aliases()
from builtins import str
from builtins import object
from past.utils import old_div
__author__ = 'Bartosz Kościów'

import random
import time
import piader_1_1.enemy as enemy
import piader_1_1.player as player
import piader_1_1.event_server as event_server
import queue
import piader_1_1.local_key as keyboard


class Piader(object):
    """Piader main class"""
    game_tick = 0.5 #1.0
    objects = []
    score = 0
    game_on = True

    def __init__(self, lcds):
        """init class"""
        if len(lcds) != 1 and len(lcds) != 2:
            raise ValueError("Must have 1 or 2 lcds")

        self.info_lcd = None
        if len(lcds) == 2:
            self.game_lcd = lcds[0]
            self.info_lcd = lcds[1]
        if len(lcds) == 1:
            self.game_lcd = lcds[0]

        self.width = self.game_lcd.get_width()
        self.height = self.game_lcd.get_height()
        if self.width < 6:
            raise ValueError("Width must be larger than 5")
        if self.height < 3:
            raise ValueError("Height must be larger than 2")

        self.player = player.Player(
            (self.width // 2) - 2,
            self.height - 1,
            self.width,
            self.objects
        )
        self.queue = queue.Queue()
        self.event_server = event_server.EventServerThread(self.queue)
        self.local_keyboard = keyboard.Keyboard()
        self.start_game()

    def start_game(self):
        """starts game, add enemy, player and start control server"""
        self.objects.append(enemy.Enemy(2, 0, self.width, self.objects))
        self.objects.append(self.player)
        self.event_server.start()

    def tick(self):
        """game tick"""
        try:
            event = self.queue.get(True, 0.05)
            if event == "player.move.left":
                self.player.move_left()
            if event == "player.move.right":
                self.player.move_right()
            if event == "player.move.fire":
                self.player.fire()

        except queue.Empty:
            pass

        self.game_lcd.buffer_clear()
        for item in self.objects:
            item.tick()
            self.draw(item)

        if self.info_lcd:
            self.info_lcd.write(str(self.score), 0, 0)

        self.game_lcd.flush()

        self.collision_check()

    def collision_check(self):
        """checks for collisions"""
        for source in self.objects:
            if source.can_hit():
                for target in self.objects:
                    if target.is_hit(source):
                        if isinstance(target, player.Player):
                            self.game_over()
                        if isinstance(target, enemy.Enemy):
                            self.enemy_hit(source, target)
                            self.score += 1

    def draw(self, item):
        """draw sprite on screen"""
        (position_x, position_y) = item.get_position()

        if position_y >= self.height or position_y < 0:
            self.objects.remove(item)
            item.event_discard()
            return

        self.game_lcd.write(item.get_sprite(), position_x, position_y)

    def game_over(self):
        """displays game over and stops game"""
        self.game_on = False
        self.game_lcd.buffer_clear()
        if self.info_lcd:
            self.info_lcd.buffer_clear()
        self.game_lcd.write("Game Over", 2, 2)
        self.game_lcd.flush()
        self.event_server.join()

    def enemy_hit(self, source, target):
        """event enemy hit"""
        target.event_discard()
        source.event_discard()
        self.objects.remove(source)
        self.objects.remove(target)
        self.objects.append(
            enemy.Enemy(
                random.randint(1, self.width - 3),
                random.randint(0, self.height - 3),
                self.width,
                self.objects
            )
        )

    def game(self):
        """game loop"""
        try:
            while self.game_on:
                start = time.time()
                self.local_keyboard.read()
                self.tick()
                end = time.time()
                if end - start < self.game_tick:
                    t_delta = end - start
                    time.sleep(max(0, self.game_tick - t_delta))
        finally:
            self.local_keyboard.shutdown()
            self.event_server.join()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,W0231
"""Driver for lcd. Uses GPIO outputs
Examples:
l = lcd.CharLCD(20, 4, Gpio())

g = Gpio()
g.pins = {
    'RS': 24,
    'E': 17,
    'E2': None,
    'DB4': 27,
    'DB5': 25,
    'DB6': 23,
    'DB7': 22
}
l = lcd.CharLCD(20, 4, g)
"""

import RPi.GPIO as GPIO  # pylint: disable=I0011,F0401
import time
from charlcd.drivers.base import BaseDriver


class Gpio(BaseDriver):
    """Gpio LCD driver"""
    def __init__(self):
        """pins in 4bit mode"""
        self.pins = {
            'RS': 25,  # 0 ->instruction, 1->data
            'E': 24,
            'E2': None,
            'DB4': 22,
            'DB5': 23,
            'DB6': 27,
            'DB7': 17
        }
        self.data_pins = [
            'DB4',
            'DB5',
            'DB6',
            'DB7'
        ]
        self.mode = 8
        self.initialized = False

    def init(self):
        """Set gpio pins"""
        if self.initialized:
            return
        for pin in self.pins:
            if self.pins[pin] is not None:
                GPIO.setup(self.pins[pin], GPIO.OUT)
                GPIO.output(self.pins[pin], 0)

        self.initialized = True

    def cmd(self, char, enable=0):
        """Write output as command. Sets RS to 0
        Args:
            char: hex to write
        """
        GPIO.output(self.pins['RS'], 0)
        self.write(char, enable)

    def shutdown(self):
        """Clean GPIO pins"""
        GPIO.cleanup()

    def send(self, enable=0):
        """Send E signal"""
        if enable == 0:
            pin = self.pins['E']
        elif enable == 1:
            pin = self.pins['E2']

        if pin is None:
            raise IndexError("Wrong enable index")

        GPIO.output(pin, 1)
        time.sleep(0.005)
        GPIO.output(pin, 0)

    def _write8(self, char, enable=0):
        """Write 8 bits"""
        self._write(char)
        self.send(enable)

    def _write4(self, char, enable=0):
        """Prepare and write 4/4 bits"""
        data = (char >> 4)
        self._write(data)
        self.send(enable)

        data = (char & 0x0F)
        self._write(data)
        self.send(enable)

    def _write(self, data):
        """Write to gpio"""
        for i in self.data_pins:
            value = data & 0x01
            GPIO.output(self.pins[i], value)
            data >>= 1

    def write(self, char, enable=0):
        """Write char to lcd
        Args:
            char: hex char to write
        """
        if self.mode == 4:
            self._write4(char, enable)
        else:
            self._write8(char, enable)

    def char(self, char, enable=0):
        """Write char to lcd
        Args:
            char: char to write
        """
        GPIO.output(self.pins['RS'], 1)
        self.write(ord(char), enable)

    def set_mode(self, mode):
        """Set lcd mode
        Args:
            mode: 4 | 8 bit mode
        """
        self.mode = mode



#!/usr/bin/python
# -*- coding: utf-8 -*-
#pylint: disable=I0011,W0231

"""Player class"""

__author__ = 'Bartosz Kościów'
import piader_1_1.item as item
import piader_1_1.missile as missile
import piader_1_1.bomb as bomb


class Player(item.Item):
    """Player class"""
    sprite = {
        'left': "&#",
        'right': "#&"
    }
    heading = 'left'
    missile = {
        "current": 0,
        "max": 1
    }

    def __init__(self, x, y, max_x, objects):
        """init player"""
        item.Item.__init__(self, x, y)
        self.max_x = max_x
        self.objects = objects

    def move_right(self):
        """move player right"""
        if self.pos_x > self.max_x - 1:
            return
        item.Item.move_right(self)
        self.heading = "right"

    def move_left(self):
        """move player left"""
        if self.pos_x < 2:
            return
        item.Item.move_left(self)
        self.heading = "left"

    def fire(self):
        """fire a missile"""
        if self.missile['current'] < self.missile['max']:
            self.missile['current'] += 1
            if self.heading == "right":
                self.objects.append(missile.Missile(self.pos_x,
                                                    self.pos_y,
                                                    self))
            else:
                self.objects.append(missile.Missile(self.pos_x + 1,
                                                    self.pos_y,
                                                    self))

    def tick(self):
        """action in tick"""
        pass

    def get_sprite(self):
        """get sprite"""
        return self.sprite[self.heading]

    def event_discard(self):
        """discard player - not implemented"""
        pass

    def can_hit(self):
        """Player can't hit anything"""
        return False

    def is_hit(self, target):
        """Player can be hit"""
        if isinstance(target, bomb.Bomb):
            target_position = target.get_position()
            if target_position[1] == self.pos_y and \
                self.pos_x <= target_position[0] <= self.pos_x + 1:
                return True

        return False



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,R0913
"""Lcd class for buffered input

Example:
lcd = CharLCD(20,4, I2C(0x20, 1))
lcd = CharLCD(20,4, GPIO())
lcd = CharLCD(20,4, I2C(0x20, 1), 1, 1)
lcd = CharLCD(20,4, I2C(0x20, 1), cursor_blink=1)
"""
from builtins import zip  # pylint: disable=I0011,W0622
from builtins import range  # pylint: disable=I0011,W0622

import charlcd.abstract.lcd as lcd
import charlcd.abstract.buffered_interface as buffered_interface


class CharLCD(lcd.CharLCD, buffered_interface.Buffered):
    """Class for char LCD
    Uses buffered input. Data is written to buffer and flushed
    Draw is sending only those positions that changed from previous frame"""

    def __init__(self, width, height, driver,
                 cursor_visible=1, cursor_blink=1):
        lcd.CharLCD.__init__(self, width, height, driver,
                             lcd.DISPLAY_MODE_BUFFERED, cursor_visible,
                             cursor_blink)
        self.screen = []
        self.buffer = []
        self.initialized = False
        self.dirty = False

    def init(self):
        """Screen init and buffer init"""
        if self.initialized:
            return
        lcd.CharLCD.init(self)
        self.screen = [" ".ljust(self.width, " ")
                       for _ in range(0, self.height)]
        self.buffer = [" ".ljust(self.width, " ")
                       for _ in range(0, self.height)]
        self.initialized = True

    def write(self, content, pos_x=None, pos_y=None):
        """Writes content into buffer at position(x,y) or current
        Will change internal position marker to reflect string write
        Args:
            content: content to write
            pos_x: x position
            pos_y: y position
        """
        if pos_x is None:
            pos_x = self.current_pos['x']
        if pos_y is None:
            pos_y = self.current_pos['y']

        if pos_x >= self.width:
            raise IndexError
        if pos_y >= self.height:
            raise IndexError

        line = self.buffer[pos_y]
        new_line = line[0:pos_x] + content + line[pos_x + len(content):]
        line = new_line[:self.width]
        self.buffer[pos_y] = line
        self.current_pos = {'x': pos_x + len(content),
                            'y': pos_y}
        self.dirty = True

    def buffer_clear(self, from_x=None, from_y=None, width=None, height=None):
        """Clears buffer. Its not recommended to use parameters.
        Args:
            from_x: x position
            from_y: y position
            width: width of area
            height: height of area
        """
        if from_x is None and from_y is None:
            self.buffer = [" ".ljust(self.width, " ")
                           for _ in range(0, self.height)]
        else:
            if height is None:
                height = self.height - from_y
            if width is None:
                width = self.width - from_x

            for pos_y in range(from_y, from_y + height):
                line = self.buffer[pos_y]
                self.buffer[pos_y] = line[0:from_x] + \
                    "".ljust(width, " ") + line[from_x + width:]
        self.dirty = True

    def flush(self):
        """Flush buffer to screen, skips chars that didn't change"""
        if not self.dirty:
            return
        bag = list(
            zip(list(range(0, self.get_height())), self.buffer, self.screen)
        )
        for line, line_new, line_current in bag:
            if line_new != line_current:
                i = 0
                last_i = -1
                for char_new, char_current in zip(line_new, line_current):
                    if char_new != char_current:
                        if last_i != i:
                            self.driver.cmd(
                                self.get_line_address(line) + i,
                                self._get_enable(line)
                            )
                            last_i = i
                        self.driver.char(
                            char_new,
                            self._get_enable(line)
                        )
                        last_i += 1
                    i += 1
                self.screen[line] = line_new
        self.dirty = False

    def stream(self, string):
        """Stream string - use stream_char
        Args:
            string: string to display
        """
        for char in string:
            self._stream_char(char)

    def _stream_char(self, char):
        """Stream char on screen, following chars are put one after another.
        Restart from beginning after reaching end
        Args:
            char: char to display
        """
        self.write(char)
        self._next_cursor_position()

    def _next_cursor_position(self):
        """calculate next cursor position"""
        if self.current_pos['x'] >= self.width:
            self.current_pos['x'] = 0
            self.current_pos['y'] += 1
            if self.current_pos['y'] >= self.height:
                self.current_pos['y'] = 0
            self.set_xy(0, self.current_pos['y'])



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,R0913,R0902,W0231
"""Main virtual lcd (vlcd) class"""

import charlcd.abstract.lcd as lcd

DISPLAY_MODE_DIRECT = 'direct'
DISPLAY_MODE_BUFFERED = 'buffered'


class CharLCDVirtual(lcd.CharLCD):
    """
    Virtual LCD class
    Shouldn't be instanced - use lcd_virtual_buffered or lcd_virtual_direct
    """

    def __init__(self, width, height, display_mode=DISPLAY_MODE_DIRECT):
        """init virtual display"""
        self.current_pos = {
            'x': 0,
            'y': 0
        }
        self.width = width
        self.height = height
        self.display_mode = display_mode
        self.displays = []
        self.active_display = None
        self.initialized = False

    def add_display(self, pos_x, pos_y, display, offset_x=0, offset_y=0):
        """add lcd to virtual display"""
        self.displays.append({
            'x': pos_x,
            'y': pos_y,
            'offset_x': offset_x,
            'offset_y': offset_y,
            'width': display.get_width(),
            'height': display.get_height(),
            'lcd': display
        })

    def get_displays(self):
        """return all displays"""
        return self.displays

    def get_display(self, pos_x, pos_y):
        """return display for (x,y) coords"""
        if pos_x > self.width or pos_x < 0 or pos_y > self.height or pos_y < 0:
            raise IndexError

        for display in self.displays:
            if display['x'] <= pos_x < display['x'] + display['width'] -\
               display['offset_x'] and \
               display['y'] <= pos_y < display['y'] + display['height'] -\
               display['offset_y']:
                    return display

        return None

    def init(self):
        """overwrite parent function"""
        pass

    def shutdown(self):
        """call shutdown on displays"""
        for display in self.displays:
            display.lcd.shutdown()

    def stream(self, string):
        """we inherit it so we should have some body"""
        lcd.CharLCD.stream(self, string)

    def write(self, string, pos_x=None, pos_y=None):
        """we inherit it so we should have some body"""
        lcd.CharLCD.write(self, string)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#pylint: disable=I0011,W0231

"""Bomb class"""

__author__ = 'Bartosz Kościów'

import piader_1_1.item as item


class Bomb(item.Item):
    """Bomb"""
    def __init__(self, x, y, dmo):
        """init bomb"""
        item.Item.__init__(self, x, y)
        self.sprite = "@"
        self.dmo = dmo

    def tick(self):
        """action in tick - move bomb down"""
        self.move_down()

    def event_discard(self):
        """discard bomb, decrease DMO bomb counter"""
        self.dmo.bombs['current'] -= 1

    def can_hit(self):
        """bomb can hit something"""
        return True

    def is_hit(self, target):
        """bomb cant be hit"""
        return False



#!/usr/bin/python
# -*- coding: utf-8 -*-
#pylint: disable=I0011,W0231

"""Missile class"""

__author__ = 'Bartosz Kościów'

import piader_1_1.item as item


class Missile(item.Item):
    """Missile"""
    sprite = "|"

    def __init__(self, x, y, player):
        """init bomb"""
        item.Item.__init__(self, x, y)
        self.player = player

    def tick(self):
        """action in tick - move missile up"""
        self.move_up()

    def event_discard(self):
        """discard missile, decrease Player bomb counter"""
        self.player.missile['current'] -= 1

    def can_hit(self):
        """missile can hit something"""
        return True

    def is_hit(self, target):
        """missile cant be hit"""
        return False



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""test script for virtual direct lcd"""
import sys
sys.path.append("../../")
import RPi.GPIO as GPIO  # NOQA pylint: disable=I0011,F0401
from charlcd import direct as lcd  # NOQA
from charlcd.drivers.gpio import Gpio  # NOQA
from charlcd.drivers.i2c import I2C  # NOQA pylint: disable=I0011,F0401
from charlcd import virtual_direct as vlcd  # NOQA

GPIO.setmode(GPIO.BCM)


def test1():
    """demo: 16x2 + 20x4 = 20x6 top, down"""
    lcd_1 = lcd.CharLCD(16, 2, I2C(0x20, 1))
    lcd_2 = lcd.CharLCD(20, 4, Gpio(), 0, 0)
    lcd_1.init()
    lcd_2.init()

    vlcd_1 = vlcd.CharLCD(20, 6)
    vlcd_1.add_display(0, 0, lcd_2)
    vlcd_1.add_display(2, 4, lcd_1)

    vlcd_1.write('test me 123456789qwertyuiop')

    vlcd_1.set_xy(2, 0)
    vlcd_1.write('1')
    vlcd_1.write('2', 2, 1)
    vlcd_1.write('3', 2, 2)
    vlcd_1.set_xy(2, 3)
    vlcd_1.write('4')
    vlcd_1.write('5', 2, 4)
    vlcd_1.write('6', 2, 5)

    vlcd_1.set_xy(3, 4)
    vlcd_1.write('-Second blarg !')
    vlcd_1.set_xy(4, 5)
    vlcd_1.write("-second line")

    vlcd_1.set_xy(11, 3)
    vlcd_1.stream("two liner and screener")


def test2():
    """demo: 16x2 + 20x4 = 36x4 left, right"""
    lcd_1 = lcd.CharLCD(16, 2, I2C(0x20, 1))
    lcd_2 = lcd.CharLCD(20, 4, Gpio(), 0, 0)
    lcd_1.init()
    lcd_2.init()
    vlcd_1 = vlcd.CharLCD(36, 4)
    vlcd_1.add_display(0, 0, lcd_2)
    vlcd_1.add_display(20, 0, lcd_1)
    vlcd_1.write('test me 123456789qwertyuiopasdfghjkl12')


def test3():
    """demo: 16x2 + 20x4 = 16x6 (offset)"""
    lcd_1 = lcd.CharLCD(16, 2, I2C(0x20, 1))
    lcd_2 = lcd.CharLCD(20, 4, Gpio(), 0, 0)
    lcd_1.init()
    lcd_2.init()

    vlcd_1 = vlcd.CharLCD(16, 6)
    vlcd_1.add_display(0, 0, lcd_2, 4, 0)
    vlcd_1.add_display(0, 4, lcd_1)

    vlcd_1.set_xy(0, 0)
    vlcd_1.write('1234567890123456')
    vlcd_1.set_xy(0, 1)
    vlcd_1.write('2')
    vlcd_1.set_xy(0, 2)
    vlcd_1.write('3')
    vlcd_1.set_xy(0, 3)
    vlcd_1.write('4')
    vlcd_1.set_xy(0, 4)
    vlcd_1.write('5')
    vlcd_1.write('6', 0, 5)


def test4():
    """demo: 16x2 + 20x4 = 10x2 - offset left right """
    lcd_1 = lcd.CharLCD(16, 2, I2C(0x20, 1))
    lcd_2 = lcd.CharLCD(20, 4, Gpio(), 0, 0)
    lcd_1.init()
    lcd_2.init()

    vlcd_1 = vlcd.CharLCD(10, 2)
    vlcd_1.add_display(0, 0, lcd_1, 11)
    vlcd_1.add_display(5, 0, lcd_2)

    vlcd_1.write('1234567890123456')
    vlcd_1.write('/', 0, 1)
    vlcd_1.write('*', 9, 1)
    vlcd_1.write('-', 0, 0)
    vlcd_1.write('#', 9, 0)

    vlcd_1.set_xy(5, 1)
    vlcd_1.write('!')

    vlcd_1.set_xy(4, 1)
    vlcd_1.write('+')


def test5():
    """demo: 16x2 + 20x4 = 10x2 + 8x2 - offset left right """
    lcd_1 = lcd.CharLCD(16, 2, I2C(0x20, 1))
    lcd_2 = lcd.CharLCD(20, 4, Gpio(), 0, 0)

    vlcd_1 = vlcd.CharLCD(10, 2)
    vlcd_1.add_display(0, 0, lcd_1, 11)
    vlcd_1.add_display(5, 0, lcd_2)
    vlcd_1.init()

    vlcd_2 = vlcd.CharLCD(8, 2)
    vlcd_2.add_display(0, 0, lcd_1)
    vlcd_2.init()

    vlcd_1.write('1234567890123456')

    vlcd_1.write('/', 0, 1)
    vlcd_1.set_xy(9, 1)
    vlcd_1.write('*')
    vlcd_1.write('-', 0, 0)
    vlcd_1.set_xy(9, 0)
    vlcd_1.write('#')
    vlcd_1.write('!', 5, 1)
    vlcd_1.write('+', 4, 1)

    vlcd_2.set_xy(0, 0)
    vlcd_2.write('#')
    vlcd_2.write('#', 7, 0)
    vlcd_2.write('#', 7, 1)
    vlcd_2.write('#', 0, 1)


def test6():
    """demo: 40x4 = 20x4 + 20x4"""
    drv = I2C(0x3a, 1)
    drv.pins['E2'] = 6
    lcd_1 = lcd.CharLCD(40, 4, drv, 0, 0)
    vlcd_1 = vlcd.CharLCD(20, 4)
    vlcd_1.add_display(0, 0, lcd_1)

    vlcd_2 = vlcd.CharLCD(20, 4)
    vlcd_2.add_display(0, 0, lcd_1, 20)

    vlcd_1.init()
    vlcd_2.init()

    vlcd_1.write('-First blarg1 !')
    vlcd_1.write('-Second blarg2 !', 0, 1)
    vlcd_1.write('-Third blarg3 !', 0, 2)
    vlcd_1.write('-Fourth blarg4 !', 0, 3)

    vlcd_2.write('-First blarg1 !')
    vlcd_2.write('-Second blarg2 !', 0, 1)
    vlcd_2.write('-Third blarg3 !', 0, 2)
    vlcd_2.write('-Fourth blarg4 !', 0, 3)
    vlcd_1.set_xy(15, 2)
    vlcd_1.stream('1234567890qwertyuiopasdfghjkl')

test6()



#!/usr/bin/python
# -*- coding: utf-8 -*-
#pylint: disable=I0011,W0231

"""Game event catcher"""
from __future__ import print_function
from future import standard_library
standard_library.install_aliases()
__author__ = 'Bartosz Kościów'

import socket
from threading import Thread

TCP_IP = '0.0.0.0'
TCP_PORT = 5005
BUFFER_SIZE = 1024


class ListenerThread(Thread):
    """connection listener"""
    def __init__(self, conn, addr, queue):
        Thread.__init__(self)
        self.client = conn
        self.addr = addr
        self.client.settimeout(0.5)
        self.work = True
        self.queue = queue

    def run(self):
        """thread"""
        while self.work:
            try:
                data = self.client.recv(BUFFER_SIZE).decode("UTF-8")
                if not data:
                    break
                else:
                    self.queue.put(data)
            except socket.timeout:
                pass

        self.client.close()

    def join(self, timeout=None):
        """stop thread"""
        self.work = False
        Thread.join(self, timeout)


class EventServerThread(Thread):
    """server thread"""
    def __init__(self, queue):
        Thread.__init__(self)
        self.work = True
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.settimeout(0.5)
        self.socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        self.socket.bind((TCP_IP, TCP_PORT))
        self.socket.listen(3)
        self.threads = []
        self.queue = queue

    def run(self):
        """start server thread"""
        try:
            print("Server is listening for connections...")
            while self.work:
                try:
                    client, address = self.socket.accept()
                    listener = ListenerThread(client, address, self.queue)
                    listener.start()
                    self.threads.append(listener)
                except socket.timeout:
                    pass
        finally:
            self.socket.close()
        for thread in self.threads:
            thread.join()
        print("Server down")

    def join(self, timeout=None):
        """stop server and all listeners"""
        self.work = False
        Thread.join(self, timeout)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#pylint: disable=I0011,W0231

"""Enemy class"""

__author__ = 'Bartosz Kościów'
import random
import piader_1_1.item as item
import piader_1_1.bomb as bomb
import piader_1_1.missile as missile


class Enemy(item.Item):
    """DMO - defined moving object"""
    bombs = {
        "current": 0,
        "max": 1,
        "chance": 100
    }
    sprite = "<*>"

    def __init__(self, x, y, max_x, objects):
        """init enemy"""
        item.Item.__init__(self, x, y)
        self.max_x = max_x
        self.objects = objects

    def tick(self):
        """action in tick"""
        if self.bombs['max'] > self.bombs['current'] and \
                        random.randint(1, 100) < self.bombs['chance']:
            self.bombs['current'] += 1
            self.objects.append(bomb.Bomb(self.pos_x + 1, self.pos_y, self))

        direction = []
        if self.pos_x > 2:
            direction.append(-1)
        if self.pos_x < self.max_x - len(self.sprite) - 1:
            direction.append(1)
        self.pos_x += random.choice(direction)

    def event_discard(self):
        """discard enemy - we are discarding enemy elsewhere"""
        pass

    def can_hit(self):
        """enemy can't hit anything"""
        return False

    def is_hit(self, target):
        """enemy can be hit"""
        if isinstance(target, missile.Missile):
            target_position = target.get_position()
            if target_position[1] == self.pos_y and \
                self.pos_x <= target_position[0] <= self.pos_x + 2:
                return True

        return False



#!/usr/bin/python
# -*- coding: utf-8 -*-
# pylint: disable=I0011,R0913
"""Virtual Lcd class for buffered input

Example:
lcd = CharLCD(20,4, I2C(0x20, 1))
lcd = CharLCD(20,4, GPIO())
lcd = CharLCD(20,4, I2C(0x20, 1), 1, 1)
lcd = CharLCD(20,4, I2C(0x20, 1), cursor_blink=1)
"""
from builtins import range  # pylint: disable=I0011,W0622
import charlcd.abstract.lcd_virtual as lcd
import charlcd.abstract.buffered_interface as buffered_interface


class CharLCD(lcd.CharLCDVirtual, buffered_interface.Buffered):
    """Virtual Lcd class for buffered input"""
    def __init__(self, width, height):
        lcd.CharLCDVirtual.__init__(self,
                                    width, height,
                                    lcd.DISPLAY_MODE_BUFFERED)
        self.screen = []
        self.buffer = []

    def init(self):
        """Buffer init"""
        if self.initialized:
            return
        if len(self.displays) == 0:
            raise ValueError("Add lcd before init vlcd")

        self.buffer = [" ".ljust(self.width, " ")
                       for _ in range(0, self.height)]
        for display in self.displays:
            display['lcd'].init()
        self.initialized = True

    def write(self, content, pos_x=None, pos_y=None):
        """write to buffer"""
        if pos_x is None:
            pos_x = self.current_pos['x']
        if pos_y is None:
            pos_y = self.current_pos['y']

        if pos_x >= self.width:
            raise IndexError
        if pos_y >= self.height:
            raise IndexError

        line = self.buffer[pos_y]
        new_line = line[0:pos_x] + content + line[pos_x + len(content):]
        line = new_line[:self.width]
        self.buffer[pos_y] = line
        self.current_pos = {'x': pos_x + len(content),
                            'y': pos_y}

        for char in content:
            self._char(char, pos_x, pos_y)
            pos_x += 1

    def _char(self, char, pos_x, pos_y):
        """Write char to proper lcd buffer
        Args:
            char: char to write
        """
        if pos_x >= self.width:
            return

        self._select_active_display(pos_x, pos_y)

        if self.active_display is None:
            return

        self.active_display['lcd'].write(
            char,
            pos_x - self.active_display['x'] + self.active_display['offset_x'],
            pos_y - self.active_display['y'] + self.active_display['offset_y']
        )

        if pos_x >= self.active_display['x'] + \
                self.active_display['width'] - 1 - \
                self.active_display['offset_x']:
            self.active_display = None

    def _select_active_display(self, pos_x, pos_y):
        """Helper, look for proper lcd for given coordinates, may return None
        Args:
            pos_x: x position
            pos_y: y position
        """
        self.active_display = self.get_display(pos_x, pos_y)

    def buffer_clear(self, from_x=None, from_y=None, width=None, height=None):
        """Clears buffer. Don't use parameters, they are for compatibility only
        Args:
            from_x: x position
            from_y: y position
            width: width of area
            height: height of area
        """
        self.buffer = [" ".ljust(self.width, " ")
                       for _ in range(0, self.height)]

        for display in self.displays:
            display['lcd'].buffer_clear(
                display['offset_x'],
                display['offset_y'],
                display['width'] - display['offset_x'],
                display['height'] - display['offset_y']
            )

    def flush(self):
        """Flush buffer to screen, skips chars that didn't change"""
        for display in self.displays:
            display['lcd'].flush()

    def stream(self, string):
        """Stream string - use stream_char
        Args:
            string: string to display
        """
        for char in string:
            self._stream_char(char)

    def _stream_char(self, char):
        """Stream char on screen, following chars are put one after another.
        Restart from beginning after reaching end
        Args:
            char: char to display
        """
        self.write(char)
        self._next_cursor_position()

    def _next_cursor_position(self):
        """Calculate next position, break lines and can go back to top"""
        if self.current_pos['x'] >= self.width:
            self.current_pos['x'] = 0
            self.current_pos['y'] += 1
            self.active_display = None
            if self.current_pos['y'] >= self.height:
                self.current_pos['y'] = 0



#!/usr/bin/python
# -*- coding: utf-8 -*-
#pylint: disable=I0011,R0921

"""Item interface"""
from builtins import object
__author__ = 'Bartosz Kościów'


class Item(object):
    """Base class for all elements in game"""
    sprite = "*"

    def __init__(self, x, y):
        """init function"""
        self.pos_x = x
        self.pos_y = y

    def tick(self):
        """tick function"""
        raise NotImplementedError("tick not implemented")

    def get_sprite(self):
        """get sprite"""
        return self.sprite

    def get_position(self):
        """get position"""
        return self.pos_x, self.pos_y

    def event_discard(self):
        """called when object is discarded"""
        raise NotImplementedError("event_discard not implemented")

    def is_hit(self, target):
        """check if object is hit by target"""
        raise NotImplementedError("is_hit not implemented")

    def can_hit(self):
        """object can hit something else"""
        raise NotImplementedError("can_hit not implemented")

    def move_up(self):
        """move object up"""
        self.pos_y -= 1

    def move_down(self):
        """move object down"""
        self.pos_y += 1

    def move_right(self):
        """move object right"""
        self.pos_x += 1

    def move_left(self):
        """move object left"""
        self.pos_x -= 1



#!/usr/bin/python
# -*- coding: utf-8 -*-
"""test script for direct lcd input"""
import sys
sys.path.append("../../")
import RPi.GPIO as GPIO  # NOQA pylint: disable=I0011,F0401
from charlcd.drivers.gpio import Gpio  # NOQA
from charlcd import direct as lcd  # NOQA
from charlcd.drivers.i2c import I2C  # NOQA pylint: disable=I0011,F0401

GPIO.setmode(GPIO.BCM)


def test1():
    """demo 16x2 by i2c and 20x4 by gpio"""
    lcd_1 = lcd.CharLCD(16, 2, I2C(0x20, 1))
    lcd_1.init()
    lcd_1.write('-Second blarg !')
    lcd_1.set_xy(0, 1)
    lcd_1.write("-second line")

    lcd_2 = lcd.CharLCD(20, 4, Gpio())
    lcd_2.init()
    lcd_2.write('-  Blarg !')
    lcd_2.write('-   Grarg !', 0, 1)
    lcd_2.set_xy(0, 2)
    lcd_2.write('-    ALIVE !!!!')


def test2():
    """demo - 20x4 by gpio"""
    lcd_2 = lcd.CharLCD(20, 4, Gpio())
    lcd_2.init()
    lcd_2.write('-  Blarg !')
    lcd_2.write('-   Grarg !', 0, 1)
    lcd_2.set_xy(0, 2)
    lcd_2.write('-    ALIVE !!!!')
    lcd_2.stream('1234567890qwertyuiopasdfghjkl')


def test3():
    """demo 3 - lcd 40x4 by gpio"""
    drv = I2C(0x3a, 1)
    drv.pins['E2'] = 6
    lcd_1 = lcd.CharLCD(40, 4, drv, 0, 0)
    lcd_1.init()
    lcd_1.write('-First blarg1 !')
    lcd_1.write('-Second blarg2 !', 0, 1)
    lcd_1.write('-Third blarg3 !', 0, 2)
    lcd_1.write('-Fourth blarg4 !', 0, 3)
    lcd_1.write('12345678901234567890', 15, 1)
    lcd_1.stream('1234567890qwertyuiopasdfghjkl')


def test4():
    """demo 4 - lcd 40x4 by i2c"""
    drv = I2C(0x3a, 1)
    drv.pins['E2'] = 6
    lcd_1 = lcd.CharLCD(40, 4, drv, 0, 0)
    lcd_1.init()
    lcd_1.write('-First blarg1 !')
    lcd_1.write('-Second blarg2 !', 0, 1)
    lcd_1.write('-Third blarg3 !', 0, 2)
    lcd_1.write('-Fourth blarg4 !', 0, 3)

test3()
